@@ -2,14 +2,24 @@
 # Copyright (C) 2001,2002 Python Software Foundation
 # csv package unit tests
 
-from codecs import EncodedFile
+import array
+import decimal
 import os
+import string
 import sys
-import unittest2 as unittest
-from StringIO import StringIO
 import tempfile
+import unittest2 as unittest
+from codecs import EncodedFile
+from StringIO import StringIO
+
 import unicodecsv as csv
 
+
+# pypy and cpython differ under which exception is raised under some
+# circumstances e.g. whether a module is written in C or not.
+py_compat_exc = (TypeError, AttributeError)
+
+
 class Test_Csv(unittest.TestCase):
     """
     Test the underlying C csv parser in ways that are not appropriate
@@ -17,21 +27,21 @@ class Test_Csv(unittest.TestCase):
     in TestDialectRegistry.
     """
     def _test_arg_valid(self, ctor, arg):
-        self.assertRaises(TypeError, ctor)
-        self.assertRaises(TypeError, ctor, None)
-        self.assertRaises(TypeError, ctor, arg, bad_attr = 0)
-        self.assertRaises(TypeError, ctor, arg, delimiter = 0)
-        self.assertRaises(TypeError, ctor, arg, delimiter = 'XX')
+        self.assertRaises(py_compat_exc, ctor)
+        self.assertRaises(py_compat_exc, ctor, None)
+        self.assertRaises(py_compat_exc, ctor, arg, bad_attr=0)
+        self.assertRaises(py_compat_exc, ctor, arg, delimiter=0)
+        self.assertRaises(py_compat_exc, ctor, arg, delimiter='XX')
         self.assertRaises(csv.Error, ctor, arg, 'foo')
-        self.assertRaises(TypeError, ctor, arg, delimiter=None)
-        self.assertRaises(TypeError, ctor, arg, delimiter=1)
-        self.assertRaises(TypeError, ctor, arg, quotechar=1)
-        self.assertRaises(TypeError, ctor, arg, lineterminator=None)
-        self.assertRaises(TypeError, ctor, arg, lineterminator=1)
-        self.assertRaises(TypeError, ctor, arg, quoting=None)
-        self.assertRaises(TypeError, ctor, arg,
+        self.assertRaises(py_compat_exc, ctor, arg, delimiter=None)
+        self.assertRaises(py_compat_exc, ctor, arg, delimiter=1)
+        self.assertRaises(py_compat_exc, ctor, arg, quotechar=1)
+        self.assertRaises(py_compat_exc, ctor, arg, lineterminator=None)
+        self.assertRaises(py_compat_exc, ctor, arg, lineterminator=1)
+        self.assertRaises(py_compat_exc, ctor, arg, quoting=None)
+        self.assertRaises(py_compat_exc, ctor, arg,
                           quoting=csv.QUOTE_ALL, quotechar='')
-        self.assertRaises(TypeError, ctor, arg,
+        self.assertRaises(py_compat_exc, ctor, arg,
                           quoting=csv.QUOTE_ALL, quotechar=None)
 
     def test_reader_arg_valid(self):
@@ -52,11 +62,14 @@ class Test_Csv(unittest.TestCase):
         self.assertEqual(obj.dialect.skipinitialspace, False)
         self.assertEqual(obj.dialect.strict, False)
         # Try deleting or changing attributes (they are read-only)
-        self.assertRaises(TypeError, delattr, obj.dialect, 'delimiter')
-        self.assertRaises(TypeError, setattr, obj.dialect, 'delimiter', ':')
-        self.assertRaises(AttributeError, delattr, obj.dialect, 'quoting')
-        self.assertRaises(AttributeError, setattr, obj.dialect,
-                          'quoting', None)
+        self.assertRaises(py_compat_exc, delattr,
+                          obj.dialect, 'delimiter')
+        self.assertRaises(py_compat_exc, setattr,
+                          obj.dialect, 'delimiter', ':')
+        self.assertRaises(py_compat_exc, delattr,
+                          obj.dialect, 'quoting')
+        self.assertRaises(py_compat_exc, setattr,
+                          obj.dialect, 'quoting', None)
 
     def test_reader_attrs(self):
         self._test_default_attrs(csv.reader, [])
@@ -89,14 +102,14 @@ class Test_Csv(unittest.TestCase):
     def _test_dialect_attrs(self, ctor, *args):
         # Now try with dialect-derived options
         class dialect:
-            delimiter='-'
-            doublequote=False
-            escapechar='^'
-            lineterminator='$'
-            quotechar='#'
-            quoting=csv.QUOTE_ALL
-            skipinitialspace=True
-            strict=False
+            delimiter = '-'
+            doublequote = False
+            escapechar = '^'
+            lineterminator = '$'
+            quotechar = '#'
+            quoting = csv.QUOTE_ALL
+            skipinitialspace = True
+            strict = False
         args = args + (dialect,)
         obj = ctor(*args)
         self.assertEqual(obj.dialect.delimiter, '-')
@@ -114,7 +127,6 @@ class Test_Csv(unittest.TestCase):
     def test_writer_dialect_attrs(self):
         self._test_dialect_attrs(csv.writer, StringIO())
 
-
     def _write_test(self, fields, expect, **kwargs):
         fd, name = tempfile.mkstemp()
         fileobj = os.fdopen(fd, "w+b")
@@ -133,59 +145,68 @@ class Test_Csv(unittest.TestCase):
         self._write_test((), '')
         self._write_test([None], '""')
         self.assertRaises(csv.Error, self._write_test,
-                          [None], None, quoting = csv.QUOTE_NONE)
+                          [None], None, quoting=csv.QUOTE_NONE)
+
         # Check that exceptions are passed up the chain
         class BadList:
             def __len__(self):
-                return 10;
+                return 10
+
             def __getitem__(self, i):
                 if i > 2:
                     raise IOError
+
         self.assertRaises(IOError, self._write_test, BadList(), '')
+
         class BadItem:
             def __str__(self):
                 raise IOError
+
         self.assertRaises(IOError, self._write_test, [BadItem()], '')
 
     def test_write_bigfield(self):
         # This exercises the buffer realloc functionality
         bigstring = 'X' * 50000
-        self._write_test([bigstring,bigstring], '%s,%s' % \
+        self._write_test([bigstring, bigstring], '%s,%s' %
                          (bigstring, bigstring))
 
     def test_write_quoting(self):
-        self._write_test(['a',1,'p,q'], 'a,1,"p,q"')
+        self._write_test(['a', 1, 'p,q'], 'a,1,"p,q"')
         self.assertRaises(csv.Error,
                           self._write_test,
-                          ['a',1,'p,q'], 'a,1,p,q',
-                          quoting = csv.QUOTE_NONE)
-        self._write_test(['a',1,'p,q'], 'a,1,"p,q"',
-                         quoting = csv.QUOTE_MINIMAL)
-        self._write_test(['a',1,'p,q'], '"a",1,"p,q"',
-                         quoting = csv.QUOTE_NONNUMERIC)
-        self._write_test(['a',1,'p,q'], '"a","1","p,q"',
-                         quoting = csv.QUOTE_ALL)
-        self._write_test(['a\nb',1], '"a\nb","1"',
-                         quoting = csv.QUOTE_ALL)
+                          ['a', 1, 'p,q'], 'a,1,p,q',
+                          quoting=csv.QUOTE_NONE)
+        self._write_test(['a', 1, 'p,q'], 'a,1,"p,q"',
+                         quoting=csv.QUOTE_MINIMAL)
+        self._write_test(['a', 1, 'p,q'], '"a",1,"p,q"',
+                         quoting=csv.QUOTE_NONNUMERIC)
+        self._write_test(['a', 1, 'p,q'], '"a","1","p,q"',
+                         quoting=csv.QUOTE_ALL)
+        self._write_test(['a\nb', 1], '"a\nb","1"',
+                         quoting=csv.QUOTE_ALL)
+
+    def test_write_decimal(self):
+        self._write_test(['a', decimal.Decimal("1.1"), 'p,q'], '"a",1.1,"p,q"',
+                         quoting=csv.QUOTE_NONNUMERIC)
 
     def test_write_escape(self):
-        self._write_test(['a',1,'p,q'], 'a,1,"p,q"',
+        self._write_test(['a', 1, 'p,q'], 'a,1,"p,q"',
                          escapechar='\\')
         self.assertRaises(csv.Error,
                           self._write_test,
-                          ['a',1,'p,"q"'], 'a,1,"p,\\"q\\""',
+                          ['a', 1, 'p,"q"'], 'a,1,"p,\\"q\\""',
                           escapechar=None, doublequote=False)
-        self._write_test(['a',1,'p,"q"'], 'a,1,"p,\\"q\\""',
-                         escapechar='\\', doublequote = False)
+        self._write_test(['a', 1, 'p,"q"'], 'a,1,"p,\\"q\\""',
+                         escapechar='\\', doublequote=False)
         self._write_test(['"'], '""""',
-                         escapechar='\\', quoting = csv.QUOTE_MINIMAL)
+                         escapechar='\\', quoting=csv.QUOTE_MINIMAL)
         self._write_test(['"'], '\\"',
-                         escapechar='\\', quoting = csv.QUOTE_MINIMAL,
-                         doublequote = False)
+                         escapechar='\\', quoting=csv.QUOTE_MINIMAL,
+                         doublequote=False)
         self._write_test(['"'], '\\"',
-                         escapechar='\\', quoting = csv.QUOTE_NONE)
-        self._write_test(['a',1,'p,q'], 'a,1,p\\,q',
-                         escapechar='\\', quoting = csv.QUOTE_NONE)
+                         escapechar='\\', quoting=csv.QUOTE_NONE)
+        self._write_test(['a', 1, 'p,q'], 'a,1,p\\,q',
+                         escapechar='\\', quoting=csv.QUOTE_NONE)
 
     def test_writerows(self):
         class BrokenFile:
@@ -198,7 +219,7 @@ class Test_Csv(unittest.TestCase):
         try:
             writer = csv.writer(fileobj)
             self.assertRaises(TypeError, writer.writerows, None)
-            writer.writerows([['a','b'],['c','d']])
+            writer.writerows([['a', 'b'], ['c', 'd']])
             fileobj.seek(0)
             self.assertEqual(fileobj.read(), "a,b\r\nc,d\r\n")
         finally:
@@ -214,17 +235,17 @@ class Test_Csv(unittest.TestCase):
         self._read_test([], [])
         self._read_test([''], [[]])
         self.assertRaises(csv.Error, self._read_test,
-                          ['"ab"c'], None, strict = 1)
+                          ['"ab"c'], None, strict=1)
         # cannot handle null bytes for the moment
         self.assertRaises(csv.Error, self._read_test,
-                          ['ab\0c'], None, strict = 1)
-        self._read_test(['"ab"c'], [['abc']], doublequote = 0)
+                          ['ab\0c'], None, strict=1)
+        self._read_test(['"ab"c'], [['abc']], doublequote=0)
 
     def test_read_eol(self):
-        self._read_test(['a,b'], [['a','b']])
-        self._read_test(['a,b\n'], [['a','b']])
-        self._read_test(['a,b\r\n'], [['a','b']])
-        self._read_test(['a,b\r'], [['a','b']])
+        self._read_test(['a,b'], [['a', 'b']])
+        self._read_test(['a,b\n'], [['a', 'b']])
+        self._read_test(['a,b\r\n'], [['a', 'b']])
+        self._read_test(['a,b\r'], [['a', 'b']])
         self.assertRaises(csv.Error, self._read_test, ['a,b\rc,d'], [])
         self.assertRaises(csv.Error, self._read_test, ['a,b\nc,d'], [])
         self.assertRaises(csv.Error, self._read_test, ['a,b\r\nc,d'], [])
@@ -271,7 +292,7 @@ class Test_Csv(unittest.TestCase):
         try:
             writer = csv.writer(fileobj)
             self.assertRaises(TypeError, writer.writerows, None)
-            rows = [['a\nb','b'],['c','x\r\nd']]
+            rows = [['a\nb', 'b'], ['c', 'x\r\nd']]
             writer.writerows(rows)
             fileobj.seek(0)
             for i, row in enumerate(csv.reader(fileobj)):
@@ -280,6 +301,7 @@ class Test_Csv(unittest.TestCase):
             fileobj.close()
             os.unlink(name)
 
+
 class TestDialectRegistry(unittest.TestCase):
     def test_registry_badargs(self):
         self.assertRaises(TypeError, csv.list_dialects, None)
@@ -348,8 +370,10 @@ class TestDialectRegistry(unittest.TestCase):
     def test_dialect_apply(self):
         class testA(csv.excel):
             delimiter = "\t"
+
         class testB(csv.excel):
             delimiter = ":"
+
         class testC(csv.excel):
             delimiter = "|"
 
@@ -359,7 +383,7 @@ class TestDialectRegistry(unittest.TestCase):
             fileobj = os.fdopen(fd, "w+b")
             try:
                 writer = csv.writer(fileobj)
-                writer.writerow([1,2,3])
+                writer.writerow([1, 2, 3])
                 fileobj.seek(0)
                 self.assertEqual(fileobj.read(), "1,2,3\r\n")
             finally:
@@ -370,7 +394,7 @@ class TestDialectRegistry(unittest.TestCase):
             fileobj = os.fdopen(fd, "w+b")
             try:
                 writer = csv.writer(fileobj, testA)
-                writer.writerow([1,2,3])
+                writer.writerow([1, 2, 3])
                 fileobj.seek(0)
                 self.assertEqual(fileobj.read(), "1\t2\t3\r\n")
             finally:
@@ -381,7 +405,7 @@ class TestDialectRegistry(unittest.TestCase):
             fileobj = os.fdopen(fd, "w+b")
             try:
                 writer = csv.writer(fileobj, dialect=testB())
-                writer.writerow([1,2,3])
+                writer.writerow([1, 2, 3])
                 fileobj.seek(0)
                 self.assertEqual(fileobj.read(), "1:2:3\r\n")
             finally:
@@ -392,7 +416,7 @@ class TestDialectRegistry(unittest.TestCase):
             fileobj = os.fdopen(fd, "w+b")
             try:
                 writer = csv.writer(fileobj, dialect='testC')
-                writer.writerow([1,2,3])
+                writer.writerow([1, 2, 3])
                 fileobj.seek(0)
                 self.assertEqual(fileobj.read(), "1|2|3\r\n")
             finally:
@@ -403,7 +427,7 @@ class TestDialectRegistry(unittest.TestCase):
             fileobj = os.fdopen(fd, "w+b")
             try:
                 writer = csv.writer(fileobj, dialect=testA, delimiter=';')
-                writer.writerow([1,2,3])
+                writer.writerow([1, 2, 3])
                 fileobj.seek(0)
                 self.assertEqual(fileobj.read(), "1;2;3\r\n")
             finally:
@@ -415,11 +439,12 @@ class TestDialectRegistry(unittest.TestCase):
 
     def test_bad_dialect(self):
         # Unknown parameter
-        self.assertRaises(TypeError, csv.reader, [], bad_attr = 0)
+        self.assertRaises(TypeError, csv.reader, [], bad_attr=0)
         # Bad values
-        self.assertRaises(TypeError, csv.reader, [], delimiter = None)
-        self.assertRaises(TypeError, csv.reader, [], quoting = -1)
-        self.assertRaises(TypeError, csv.reader, [], quoting = 100)
+        self.assertRaises(TypeError, csv.reader, [], delimiter=None)
+        self.assertRaises(TypeError, csv.reader, [], quoting=-1)
+        self.assertRaises(TypeError, csv.reader, [], quoting=100)
+
 
 class TestCsvBase(unittest.TestCase):
     def readerAssertEqual(self, input, expected_result):
@@ -428,7 +453,7 @@ class TestCsvBase(unittest.TestCase):
         try:
             fileobj.write(input)
             fileobj.seek(0)
-            reader = csv.reader(fileobj, dialect = self.dialect)
+            reader = csv.reader(fileobj, dialect=self.dialect)
             fields = list(reader)
             self.assertEqual(fields, expected_result)
         finally:
@@ -439,7 +464,7 @@ class TestCsvBase(unittest.TestCase):
         fd, name = tempfile.mkstemp()
         fileobj = os.fdopen(fd, "w+b")
         try:
-            writer = csv.writer(fileobj, dialect = self.dialect)
+            writer = csv.writer(fileobj, dialect=self.dialect)
             writer.writerows(input)
             fileobj.seek(0)
             self.assertEqual(fileobj.read(), expected_result)
@@ -447,6 +472,7 @@ class TestCsvBase(unittest.TestCase):
             fileobj.close()
             os.unlink(name)
 
+
 class TestDialectExcel(TestCsvBase):
     dialect = 'excel'
 
@@ -454,7 +480,7 @@ class TestDialectExcel(TestCsvBase):
         self.readerAssertEqual('abc', [['abc']])
 
     def test_simple(self):
-        self.readerAssertEqual('1,2,3,4,5', [['1','2','3','4','5']])
+        self.readerAssertEqual('1,2,3,4,5', [['1', '2', '3', '4', '5']])
 
     def test_blankline(self):
         self.readerAssertEqual('', [])
@@ -466,10 +492,10 @@ class TestDialectExcel(TestCsvBase):
         self.readerAssertEqual('""', [['']])
 
     def test_singlequoted_left_empty(self):
-        self.readerAssertEqual('"",', [['','']])
+        self.readerAssertEqual('"",', [['', '']])
 
     def test_singlequoted_right_empty(self):
-        self.readerAssertEqual(',""', [['','']])
+        self.readerAssertEqual(',""', [['', '']])
 
     def test_single_quoted_quote(self):
         self.readerAssertEqual('""""', [['"']])
@@ -506,7 +532,8 @@ class TestDialectExcel(TestCsvBase):
                                  '5', '6']])
 
     def test_quoted_quote(self):
-        self.readerAssertEqual('1,2,3,"""I see,"" said the blind man","as he picked up his hammer and saw"',
+        value = '1,2,3,"""I see,"" said the blind man","as he picked up his hammer and saw"'
+        self.readerAssertEqual(value,
                                [['1', '2', '3',
                                  '"I see," said the blind man',
                                  'as he picked up his hammer and saw']])
@@ -521,7 +548,7 @@ hammer and saw"
                                [['1', '2', '3',
                                    '"I see,"\nsaid the blind man',
                                    'as he picked up his\nhammer and saw'],
-                                ['9','8','7','6']])
+                                ['9', '8', '7', '6']])
 
     def test_dubious_quote(self):
         self.readerAssertEqual('12,12,1",', [['12', '12', '1"', '']])
@@ -533,21 +560,27 @@ hammer and saw"
         self.writerAssertEqual([['abc']], 'abc\r\n')
 
     def test_simple_writer(self):
-        self.writerAssertEqual([[1, 2, 'abc', 3, 4]], '1,2,abc,3,4\r\n')
+        self.writerAssertEqual([[1, 2, 'abc', 3, 4]],
+                               '1,2,abc,3,4\r\n')
 
     def test_quotes(self):
-        self.writerAssertEqual([[1, 2, 'a"bc"', 3, 4]], '1,2,"a""bc""",3,4\r\n')
+        self.writerAssertEqual([[1, 2, 'a"bc"', 3, 4]],
+                               '1,2,"a""bc""",3,4\r\n')
 
     def test_quote_fieldsep(self):
-        self.writerAssertEqual([['abc,def']], '"abc,def"\r\n')
+        self.writerAssertEqual([['abc,def']],
+                               '"abc,def"\r\n')
 
     def test_newlines(self):
-        self.writerAssertEqual([[1, 2, 'a\nbc', 3, 4]], '1,2,"a\nbc",3,4\r\n')
+        self.writerAssertEqual([[1, 2, 'a\nbc', 3, 4]],
+                               '1,2,"a\nbc",3,4\r\n')
+
 
 class EscapedExcel(csv.excel):
     quoting = csv.QUOTE_NONE
     escapechar = '\\'
 
+
 class TestEscapedExcel(TestCsvBase):
     dialect = EscapedExcel()
 
@@ -557,10 +590,12 @@ class TestEscapedExcel(TestCsvBase):
     def test_read_escape_fieldsep(self):
         self.readerAssertEqual('abc\\,def\r\n', [['abc,def']])
 
+
 class QuotedEscapedExcel(csv.excel):
     quoting = csv.QUOTE_NONNUMERIC
     escapechar = '\\'
 
+
 class TestQuotedEscapedExcel(TestCsvBase):
     dialect = QuotedEscapedExcel()
 
@@ -570,25 +605,38 @@ class TestQuotedEscapedExcel(TestCsvBase):
     def test_read_escape_fieldsep(self):
         self.readerAssertEqual('"abc\\,def"\r\n', [['abc,def']])
 
+
 class TestDictFields(unittest.TestCase):
-    ### "long" means the row is longer than the number of fieldnames
-    ### "short" means there are fewer elements in the row than fieldnames
+    # "long" means the row is longer than the number of fieldnames
+    # "short" means there are fewer elements in the row than fieldnames
     def test_write_simple_dict(self):
         fd, name = tempfile.mkstemp()
         fileobj = open(name, 'w+b')
         try:
-            writer = csv.DictWriter(fileobj, fieldnames = ["f1", "f2", "f3"])
+            writer = csv.DictWriter(fileobj, fieldnames=["f1", "f2", "f3"])
             writer.writeheader()
             fileobj.seek(0)
             self.assertEqual(fileobj.readline(), "f1,f2,f3\r\n")
             writer.writerow({"f1": 10, "f3": "abc"})
             fileobj.seek(0)
-            fileobj.readline() # header
+            fileobj.readline()  # header
             self.assertEqual(fileobj.read(), "10,,abc\r\n")
         finally:
             fileobj.close()
             os.unlink(name)
 
+    def test_write_unicode_header_dict(self):
+        fd, name = tempfile.mkstemp()
+        fileobj = open(name, 'w+b')
+        try:
+            writer = csv.DictWriter(fileobj, fieldnames=[u"ñ", u"ö"])
+            writer.writeheader()
+            fileobj.seek(0)
+            self.assertEqual(fileobj.readline().decode('utf-8'), u"ñ,ö\r\n")
+        finally:
+            fileobj.close()
+            os.unlink(name)
+
     def test_write_no_fields(self):
         fileobj = StringIO()
         self.assertRaises(TypeError, csv.DictWriter, fileobj)
@@ -601,7 +649,8 @@ class TestDictFields(unittest.TestCase):
             fileobj.seek(0)
             reader = csv.DictReader(fileobj,
                                     fieldnames=["f1", "f2", "f3"])
-            self.assertEqual(reader.next(), {"f1": '1', "f2": '2', "f3": 'abc'})
+            self.assertEqual(reader.next(),
+                             {"f1": '1', "f2": '2', "f3": 'abc'})
         finally:
             fileobj.close()
             os.unlink(name)
@@ -613,8 +662,10 @@ class TestDictFields(unittest.TestCase):
             fileobj.write("f1,f2,f3\r\n1,2,abc\r\n")
             fileobj.seek(0)
             reader = csv.DictReader(fileobj)
-            self.assertEqual(reader.fieldnames, ["f1", "f2", "f3"])
-            self.assertEqual(reader.next(), {"f1": '1', "f2": '2', "f3": 'abc'})
+            self.assertEqual(reader.fieldnames,
+                             ["f1", "f2", "f3"])
+            self.assertEqual(reader.next(),
+                             {"f1": '1', "f2": '2', "f3": 'abc'})
         finally:
             fileobj.close()
             os.unlink(name)
@@ -628,8 +679,10 @@ class TestDictFields(unittest.TestCase):
             f.write("f1,f2,f3\r\n1,2,abc\r\n")
             f.seek(0)
             reader = csv.DictReader(f, fieldnames=csv.reader(f).next())
-            self.assertEqual(reader.fieldnames, ["f1", "f2", "f3"])
-            self.assertEqual(reader.next(), {"f1": '1', "f2": '2', "f3": 'abc'})
+            self.assertEqual(reader.fieldnames,
+                             ["f1", "f2", "f3"])
+            self.assertEqual(reader.next(),
+                             {"f1": '1', "f2": '2', "f3": 'abc'})
         finally:
             f.close()
             os.unlink(name)
@@ -726,7 +779,7 @@ class TestDictFields(unittest.TestCase):
                                          "s2": 'def'})
 
     def test_read_with_blanks(self):
-        reader = csv.DictReader(["1,2,abc,4,5,6\r\n","\r\n",
+        reader = csv.DictReader(["1,2,abc,4,5,6\r\n", "\r\n",
                                  "1,2,abc,4,5,6\r\n"],
                                 fieldnames="1 2 3 4 5 6".split())
         self.assertEqual(reader.next(), {"1": '1', "2": '2', "3": 'abc',
@@ -741,9 +794,12 @@ class TestDictFields(unittest.TestCase):
         self.assertEqual(reader.next(), {"1": '1', "2": '2', "3": 'abc',
                                          "4": '4', "5": '5', "6": '6'})
 
+    def test_empty_file(self):
+        csv.DictReader(StringIO())
+
+
 class TestArrayWrites(unittest.TestCase):
     def test_int_write(self):
-        import array
         contents = [(20-i) for i in range(20)]
         a = array.array('i', contents)
 
@@ -798,7 +854,6 @@ class TestArrayWrites(unittest.TestCase):
             os.unlink(name)
 
     def test_char_write(self):
-        import array, string
         a = array.array('c', string.letters)
         fd, name = tempfile.mkstemp()
         fileobj = os.fdopen(fd, "w+b")
@@ -819,7 +874,7 @@ class TestUnicode(unittest.TestCase):
                                  "Marc André Lemburg,"
                                  "Guido van Rossum,"
                                  "François Pinard\r\n"),
-                                 data_encoding='iso-8859-1')
+                        data_encoding='iso-8859-1')
         reader = csv.reader(f)
         self.assertEqual(list(reader), [[u"Martin von Löwis",
                                          u"Marc André Lemburg",
@@ -836,7 +891,8 @@ class TestUnicodeErrors(unittest.TestCase):
 
     def test_encode_error_dictwriter(self):
         fd = StringIO()
-        dw = csv.DictWriter(fd, ['col1'], encoding='cp1252', errors='xmlcharrefreplace')
+        dw = csv.DictWriter(fd, ['col1'],
+                            encoding='cp1252', errors='xmlcharrefreplace')
         dw.writerow({'col1': unichr(2604)})
         self.assertEqual(fd.getvalue(), '&#2604;\r\n')
 
